@@ -12,26 +12,29 @@ from rx import Observable
 
 from dagster import check, seven
 from dagster.config import Field, Permissive
-from dagster.core.definitions.pipeline import ExecutionSelector, PipelineDefinition
+from dagster.core.definitions.pipeline import PipelineDefinition, PipelineSubsetDefinition
 from dagster.core.errors import (
     DagsterInvalidConfigError,
     DagsterInvariantViolationError,
     DagsterRunAlreadyExists,
     DagsterRunConflict,
 )
+from dagster.core.storage.migration.utils import upgrading_instance
 from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus, PipelineRunsFilter
+from dagster.core.utils import str_format_list
 from dagster.serdes import ConfigurableClass, whitelist_for_serdes
 from dagster.seven import get_current_datetime_in_utc
 from dagster.utils.merger import merge_dicts
 from dagster.utils.yaml_utils import load_yaml_from_globs
 
 from .config import DAGSTER_CONFIG_YAML_FILENAME
-from .ref import InstanceRef
+from .ref import InstanceRef, compute_logs_directory
 
 # 'airflow_execution_date' and 'is_airflow_ingest_pipeline' are hardcoded tags used in the
 # airflow ingestion logic (see: dagster_pipeline_factory.py). 'airflow_execution_date' stores the
 # 'execution_date' used in Airflow operator execution and 'is_airflow_ingest_pipeline' determines
 # whether 'airflow_execution_date' is needed.
+# https://github.com/dagster-io/dagster/issues/2403
 AIRFLOW_EXECUTION_DATE_STR = 'airflow_execution_date'
 IS_AIRFLOW_INGEST_PIPELINE_STR = 'is_airflow_ingest_pipeline'
 
@@ -198,7 +201,7 @@ class DagsterInstance:
             schedule_storage, 'schedule_storage', ScheduleStorage
         )
         self._scheduler = check.opt_inst_param(scheduler, 'scheduler', Scheduler)
-        self._run_launcher = check.opt_inst_param(run_launcher, 'run_launcher', RunLauncher)
+        self._run_launcher = check.inst_param(run_launcher, 'run_launcher', RunLauncher)
         self._dagit_settings = check.opt_dict_param(dagit_settings, 'dagit_settings')
         self._telemetry_settings = check.opt_dict_param(telemetry_settings, 'telemetry_settings')
 
@@ -210,6 +213,7 @@ class DagsterInstance:
 
     @staticmethod
     def ephemeral(tempdir=None):
+        from dagster.core.launcher.sync_in_memory_run_launcher import SyncInMemoryRunLauncher
         from dagster.core.storage.event_log import InMemoryEventLogStorage
         from dagster.core.storage.root import LocalArtifactStorage
         from dagster.core.storage.runs import InMemoryRunStorage
@@ -224,6 +228,7 @@ class DagsterInstance:
             run_storage=InMemoryRunStorage(),
             event_storage=InMemoryEventLogStorage(),
             compute_log_manager=NoOpComputeLogManager(),
+            run_launcher=SyncInMemoryRunLauncher(),
         )
 
     @staticmethod
@@ -389,11 +394,16 @@ class DagsterInstance:
             return dagster_telemetry_enabled_default
 
     def upgrade(self, print_fn=lambda _: None):
-        print_fn('Updating run storage...')
-        self._run_storage.upgrade()
+        with upgrading_instance(self):
 
-        print_fn('Updating event storage...')
-        self._event_storage.upgrade()
+            print_fn('Updating run storage...')
+            self._run_storage.upgrade()
+
+            print_fn('Updating event storage...')
+            self._event_storage.upgrade()
+
+            print_fn('Updating schedule storage...')
+            self._schedule_storage.upgrade()
 
     def dispose(self):
         self._run_storage.dispose()
@@ -410,6 +420,22 @@ class DagsterInstance:
     def has_pipeline_snapshot(self, snapshot_id):
         return self._run_storage.has_pipeline_snapshot(snapshot_id)
 
+    def get_historical_pipeline(self, snapshot_id):
+        from dagster.core.host_representation import HistoricalPipeline
+
+        snapshot = self._run_storage.get_pipeline_snapshot(snapshot_id)
+        parent_snapshot = (
+            self._run_storage.get_pipeline_snapshot(snapshot.lineage_snapshot.parent_snapshot_id)
+            if snapshot.lineage_snapshot
+            else None
+        )
+        return HistoricalPipeline(
+            self._run_storage.get_pipeline_snapshot(snapshot_id), snapshot_id, parent_snapshot
+        )
+
+    def has_historical_pipeline(self, snapshot_id):
+        return self._run_storage.has_pipeline_snapshot(snapshot_id)
+
     def get_execution_plan_snapshot(self, snapshot_id):
         return self._run_storage.get_execution_plan_snapshot(snapshot_id)
 
@@ -432,12 +458,13 @@ class DagsterInstance:
         run_id=None,
         environment_dict=None,
         mode=None,
-        solid_subset=None,
+        solids_to_execute=None,
         step_keys_to_execute=None,
         status=None,
         tags=None,
         root_run_id=None,
         parent_run_id=None,
+        solid_selection=None,
     ):
         from dagster.core.execution.api import create_execution_plan
         from dagster.core.execution.plan.plan import ExecutionPlan
@@ -446,8 +473,30 @@ class DagsterInstance:
         check.inst_param(pipeline_def, 'pipeline_def', PipelineDefinition)
         check.opt_inst_param(execution_plan, 'execution_plan', ExecutionPlan)
 
-        if solid_subset:
-            pipeline_def = pipeline_def.build_sub_pipeline(solid_subset=solid_subset)
+        # note that solids_to_execute is required to execute the solid subset, which is the
+        # frozenset version of the previous solid_subset.
+        # solid_selection is not required and will not be converted to solids_to_execute here.
+        # i.e. this function doesn't handle solid queries.
+        # solid_selection is only used to pass the user queries further down.
+        check.opt_set_param(solids_to_execute, 'solids_to_execute', of_type=str)
+        check.opt_list_param(solid_selection, 'solid_selection', of_type=str)
+
+        if solids_to_execute:
+            if isinstance(pipeline_def, PipelineSubsetDefinition):
+                # for the case when pipeline_def is created by ExecutablePipeline or ExternalPipeline
+                check.invariant(
+                    solids_to_execute == pipeline_def.solids_to_execute,
+                    'Cannot create a PipelineRun from pipeline subset {pipeline_solids_to_execute} '
+                    'that conflicts with solids_to_execute arg {solids_to_execute}'.format(
+                        pipeline_solids_to_execute=str_format_list(pipeline_def.solids_to_execute),
+                        solids_to_execute=str_format_list(solids_to_execute),
+                    ),
+                )
+            else:
+                # for cases when `create_run_for_pipeline` is directly called
+                pipeline_def = pipeline_def.get_pipeline_subset_def(
+                    solids_to_execute=solids_to_execute
+                )
 
         if execution_plan is None:
             execution_plan = create_execution_plan(
@@ -462,7 +511,8 @@ class DagsterInstance:
             run_id=run_id,
             environment_dict=environment_dict,
             mode=check.opt_str_param(mode, 'mode', default=pipeline_def.get_default_mode_name()),
-            solid_subset=solid_subset,
+            solid_selection=solid_selection,
+            solids_to_execute=solids_to_execute,
             step_keys_to_execute=step_keys_to_execute,
             status=status,
             tags=tags,
@@ -472,24 +522,28 @@ class DagsterInstance:
             execution_plan_snapshot=snapshot_from_execution_plan(
                 execution_plan, pipeline_def.get_pipeline_snapshot_id()
             ),
+            parent_pipeline_snapshot=pipeline_def.get_parent_pipeline_snapshot(),
         )
 
     def _construct_run_with_snapshots(
         self,
-        pipeline_name=None,
-        run_id=None,
-        environment_dict=None,
-        mode=None,
-        solid_subset=None,
-        step_keys_to_execute=None,
-        status=None,
-        tags=None,
-        root_run_id=None,
-        parent_run_id=None,
-        pipeline_snapshot=None,
-        execution_plan_snapshot=None,
+        pipeline_name,
+        run_id,
+        environment_dict,
+        mode,
+        solids_to_execute,
+        step_keys_to_execute,
+        status,
+        tags,
+        root_run_id,
+        parent_run_id,
+        pipeline_snapshot,
+        execution_plan_snapshot,
+        parent_pipeline_snapshot,
+        solid_selection=None,
     ):
 
+        # https://github.com/dagster-io/dagster/issues/2403
         if tags and IS_AIRFLOW_INGEST_PIPELINE_STR in tags:
             if AIRFLOW_EXECUTION_DATE_STR not in tags:
                 tags[AIRFLOW_EXECUTION_DATE_STR] = get_current_datetime_in_utc().isoformat()
@@ -499,7 +553,8 @@ class DagsterInstance:
             run_id=run_id,
             environment_dict=environment_dict,
             mode=mode,
-            solid_subset=solid_subset,
+            solid_selection=solid_selection,
+            solids_to_execute=solids_to_execute,
             step_keys_to_execute=step_keys_to_execute,
             status=status,
             tags=tags,
@@ -510,13 +565,29 @@ class DagsterInstance:
         if pipeline_snapshot is not None:
             from dagster.core.snap import create_pipeline_snapshot_id
 
-            pipeline_snapshot_id = create_pipeline_snapshot_id(pipeline_snapshot)
+            if pipeline_snapshot.lineage_snapshot:
+                if not self._run_storage.has_pipeline_snapshot(
+                    pipeline_snapshot.lineage_snapshot.parent_snapshot_id
+                ):
+                    check.invariant(
+                        create_pipeline_snapshot_id(parent_pipeline_snapshot)
+                        == pipeline_snapshot.lineage_snapshot.parent_snapshot_id,
+                        'Parent pipeline snapshot id out of sync with passed parent pipeline snapshot',
+                    )
 
+                    returned_pipeline_snapshot_id = self._run_storage.add_pipeline_snapshot(
+                        parent_pipeline_snapshot
+                    )
+                    check.invariant(
+                        pipeline_snapshot.lineage_snapshot.parent_snapshot_id
+                        == returned_pipeline_snapshot_id
+                    )
+
+            pipeline_snapshot_id = create_pipeline_snapshot_id(pipeline_snapshot)
             if not self._run_storage.has_pipeline_snapshot(pipeline_snapshot_id):
                 returned_pipeline_snapshot_id = self._run_storage.add_pipeline_snapshot(
                     pipeline_snapshot
                 )
-
                 check.invariant(pipeline_snapshot_id == returned_pipeline_snapshot_id)
 
             pipeline_run = pipeline_run.with_pipeline_snapshot_id(pipeline_snapshot_id)
@@ -552,30 +623,29 @@ class DagsterInstance:
 
     def create_run(
         self,
-        pipeline_name=None,
-        run_id=None,
-        environment_dict=None,
-        mode=None,
-        solid_subset=None,
-        step_keys_to_execute=None,
-        status=None,
-        tags=None,
-        root_run_id=None,
-        parent_run_id=None,
-        pipeline_snapshot=None,
-        execution_plan_snapshot=None,
+        pipeline_name,
+        run_id,
+        environment_dict,
+        mode,
+        solids_to_execute,
+        step_keys_to_execute,
+        status,
+        tags,
+        root_run_id,
+        parent_run_id,
+        pipeline_snapshot,
+        execution_plan_snapshot,
+        parent_pipeline_snapshot,
+        solid_selection=None,
     ):
 
-        # This logic is factored out since it's shared with `get_or_create_run`, which will soon be
-        # deleted. When it is deleted, the body of `_construct_run_with_snapshots` can be brought
-        # back into this function.
-        # https://github.com/dagster-io/dagster/issues/2412
         pipeline_run = self._construct_run_with_snapshots(
             pipeline_name=pipeline_name,
             run_id=run_id,
             environment_dict=environment_dict,
             mode=mode,
-            solid_subset=solid_subset,
+            solid_selection=solid_selection,
+            solids_to_execute=solids_to_execute,
             step_keys_to_execute=step_keys_to_execute,
             status=status,
             tags=tags,
@@ -583,23 +653,25 @@ class DagsterInstance:
             parent_run_id=parent_run_id,
             pipeline_snapshot=pipeline_snapshot,
             execution_plan_snapshot=execution_plan_snapshot,
+            parent_pipeline_snapshot=parent_pipeline_snapshot,
         )
         return self._run_storage.add_run(pipeline_run)
 
-    def get_or_create_run(
+    def register_managed_run(
         self,
-        pipeline_name=None,
-        run_id=None,
-        environment_dict=None,
-        mode=None,
-        solid_subset=None,
-        step_keys_to_execute=None,
-        status=None,
-        tags=None,
-        root_run_id=None,
-        parent_run_id=None,
-        pipeline_snapshot=None,
-        execution_plan_snapshot=None,
+        pipeline_name,
+        run_id,
+        environment_dict,
+        mode,
+        solids_to_execute,
+        step_keys_to_execute,
+        tags,
+        root_run_id,
+        parent_run_id,
+        pipeline_snapshot,
+        execution_plan_snapshot,
+        parent_pipeline_snapshot,
+        solid_selection=None,
     ):
         # The usage of this method is limited to dagster-airflow, specifically in Dagster
         # Operators that are executed in Airflow. Because a common workflow in Airflow is to
@@ -617,14 +689,16 @@ class DagsterInstance:
             run_id=run_id,
             environment_dict=environment_dict,
             mode=mode,
-            solid_subset=solid_subset,
+            solid_selection=solid_selection,
+            solids_to_execute=solids_to_execute,
             step_keys_to_execute=step_keys_to_execute,
-            status=status,
+            status=PipelineRunStatus.MANAGED,
             tags=tags,
             root_run_id=root_run_id,
             parent_run_id=parent_run_id,
             pipeline_snapshot=pipeline_snapshot,
             execution_plan_snapshot=execution_plan_snapshot,
+            parent_pipeline_snapshot=parent_pipeline_snapshot,
         )
 
         def get_run():
@@ -664,6 +738,9 @@ class DagsterInstance:
     def get_runs_count(self, filters=None):
         return self._run_storage.get_runs_count(filters)
 
+    def get_run_groups(self, filters=None, cursor=None, limit=None):
+        return self._run_storage.get_run_groups(filters=filters, cursor=cursor, limit=limit)
+
     def wipe(self):
         self._run_storage.wipe()
         self._event_storage.wipe()
@@ -785,7 +862,7 @@ class DagsterInstance:
 
     # Run launcher
 
-    def launch_run(self, run_id):
+    def launch_run(self, run_id, external_pipeline):
         '''Launch a pipeline run.
 
         This method delegates to the ``RunLauncher``, if any, configured on the instance, and will
@@ -798,69 +875,63 @@ class DagsterInstance:
             run_id (str): The id of the run the launch.
         '''
         run = self.get_run_by_id(run_id)
-        return self._run_launcher.launch_run(self, run)
+        return self._run_launcher.launch_run(self, run, external_pipeline=external_pipeline)
 
     # Scheduler
 
-    def reconcile_scheduler_state(self, python_path, repository_path, repository):
-        return self._scheduler.reconcile_scheduler_state(
-            self, repository, python_path, repository_path
-        )
+    def reconcile_scheduler_state(self, external_repository):
+        return self._scheduler.reconcile_scheduler_state(self, external_repository)
 
-    def start_schedule_and_update_storage_state(self, repository, schedule_name):
-        return self._scheduler.start_schedule_and_update_storage_state(
-            self, repository, schedule_name
-        )
+    def start_schedule_and_update_storage_state(self, external_schedule):
+        return self._scheduler.start_schedule_and_update_storage_state(self, external_schedule)
 
-    def stop_schedule_and_update_storage_state(self, repository, schedule_name):
-        return self._scheduler.stop_schedule_and_update_storage_state(
-            self, repository, schedule_name
-        )
+    def stop_schedule_and_update_storage_state(self, schedule_origin_id):
+        return self._scheduler.stop_schedule_and_update_storage_state(self, schedule_origin_id)
 
-    def stop_schedule_and_delete_from_storage(self, repository, schedule_name):
-        return self._scheduler.stop_schedule_and_delete_from_storage(
-            self, repository, schedule_name
-        )
+    def stop_schedule_and_delete_from_storage(self, schedule_origin_id):
+        return self._scheduler.stop_schedule_and_delete_from_storage(self, schedule_origin_id)
 
-    def running_schedule_count(self, repository_name, schedule_name):
-        return self._scheduler.running_schedule_count(repository_name, schedule_name)
+    def running_schedule_count(self, schedule_origin_id):
+        if self._scheduler:
+            return self._scheduler.running_schedule_count(schedule_origin_id)
+        return 0
 
     def scheduler_debug_info(self):
         from dagster.core.scheduler import SchedulerDebugInfo, ScheduleStatus
 
         errors = []
 
-        schedule_info = self.all_schedules_info()
         schedules = []
-        for repository_name, schedule in schedule_info:
-            if schedule.status == ScheduleStatus.RUNNING and not self.running_schedule_count(
-                repository_name, schedule.name
+        for schedule_state in self.all_stored_schedule_state():
+            if schedule_state.status == ScheduleStatus.RUNNING and not self.running_schedule_count(
+                schedule_state.schedule_origin_id
             ):
                 errors.append(
                     "Schedule {schedule_name} is set to be running, but the scheduler is not "
-                    "running the schedule.".format(schedule_name=schedule.name)
+                    "running the schedule.".format(schedule_name=schedule_state.name)
                 )
-            elif schedule.status == ScheduleStatus.STOPPED and self.running_schedule_count(
-                repository_name, schedule.name
+            elif schedule_state.status == ScheduleStatus.STOPPED and self.running_schedule_count(
+                schedule_state.schedule_origin_id
             ):
                 errors.append(
                     "Schedule {schedule_name} is set to be stopped, but the scheduler is still running "
-                    "the schedule.".format(schedule_name=schedule.name)
+                    "the schedule.".format(schedule_name=schedule_state.name)
                 )
 
-            if self.running_schedule_count(repository_name, schedule.name) > 1:
+            if self.running_schedule_count(schedule_state.schedule_origin_id) > 1:
                 errors.append(
                     "Duplicate jobs found: More than one job for schedule {schedule_name} are "
-                    "running on the scheduler.".format(schedule_name=schedule.name)
+                    "running on the scheduler.".format(schedule_name=schedule_state.name)
                 )
 
             schedule_info = {
-                schedule.name: {
-                    "status": schedule.status.value,
-                    "cron_schedule": schedule.cron_schedule,
-                    "python_path": schedule.python_path,
-                    "repository_name": repository_name,
-                    "repository_path": schedule.repository_path,
+                schedule_state.name: {
+                    "status": schedule_state.status.value,
+                    "cron_schedule": schedule_state.cron_schedule,
+                    "python_path": schedule_state.pipeline_origin.executable_path,
+                    "repository_pointer": schedule_state.pipeline_origin.get_repo_pointer().describe(),
+                    "schedule_origin_id": schedule_state.schedule_origin_id,
+                    "repository_origin_id": schedule_state.repository_origin_id,
                 }
             }
 
@@ -875,35 +946,32 @@ class DagsterInstance:
 
     # Schedule Storage
 
-    def create_schedule_tick(self, repository, schedule_tick_data):
-        return self._schedule_storage.create_schedule_tick(repository, schedule_tick_data)
+    def create_schedule_tick(self, schedule_tick_data):
+        return self._schedule_storage.create_schedule_tick(schedule_tick_data)
 
-    def update_schedule_tick(self, repository, tick):
-        return self._schedule_storage.update_schedule_tick(repository, tick)
+    def update_schedule_tick(self, tick):
+        return self._schedule_storage.update_schedule_tick(tick)
 
-    def get_schedule_ticks_by_schedule(self, repository, schedule_name):
-        return self._schedule_storage.get_schedule_ticks_by_schedule(repository, schedule_name)
+    def get_schedule_ticks(self, schedule_origin_id):
+        return self._schedule_storage.get_schedule_ticks(schedule_origin_id)
 
-    def get_schedule_tick_stats_by_schedule(self, repository, schedule_name):
-        return self._schedule_storage.get_schedule_tick_stats_by_schedule(repository, schedule_name)
+    def get_schedule_tick_stats(self, schedule_origin_id):
+        return self._schedule_storage.get_schedule_tick_stats(schedule_origin_id)
 
-    def all_schedules_info(self):
-        return self._schedule_storage.all_schedules_info()
+    def all_stored_schedule_state(self, repository_origin_id=None):
+        return self._schedule_storage.all_stored_schedule_state(repository_origin_id)
 
-    def all_schedules(self, repository=None):
-        return self._schedule_storage.all_schedules(repository)
+    def get_schedule_state(self, schedule_origin_id):
+        return self._schedule_storage.get_schedule_state(schedule_origin_id)
 
-    def get_schedule_by_name(self, repository, schedule_name):
-        return self._schedule_storage.get_schedule_by_name(repository, schedule_name)
+    def add_schedule_state(self, schedule_state):
+        return self._schedule_storage.add_schedule_state(schedule_state)
 
-    def add_schedule(self, repository, schedule):
-        return self._schedule_storage.add_schedule(repository, schedule)
+    def update_schedule_state(self, schedule_state):
+        return self._schedule_storage.update_schedule_state(schedule_state)
 
-    def update_schedule(self, repository, schedule):
-        return self._schedule_storage.update_schedule(repository, schedule)
-
-    def delete_schedule(self, repository, schedule):
-        return self._schedule_storage.delete_schedule(repository, schedule)
+    def delete_schedule_state(self, schedule_origin_id):
+        return self._schedule_storage.delete_schedule_state(schedule_origin_id)
 
     def wipe_all_schedules(self):
         if self._scheduler:
@@ -911,8 +979,5 @@ class DagsterInstance:
 
         self._schedule_storage.wipe()
 
-    def logs_directory_for_schedule(self, repository, schedule_name):
-        return self._scheduler.get_logs_directory(self, repository, schedule_name)
-
-    def logs_path_for_schedule(self, repository, schedule_name):
-        return self._scheduler.get_logs_path(self, repository, schedule_name)
+    def logs_path_for_schedule(self, schedule_origin_id):
+        return self._scheduler.get_logs_path(self, schedule_origin_id)
