@@ -24,7 +24,7 @@ from dagster.core.definitions.executable import InMemoryExecutablePipeline
 from dagster.core.definitions.logger import LoggerDefinition
 from dagster.core.definitions.resource import ScopedResourcesBuilder
 from dagster.core.definitions.solid import ISolidDefinition
-from dagster.core.execution.api import RunConfig, create_execution_plan, scoped_pipeline_context
+from dagster.core.execution.api import create_execution_plan, scoped_pipeline_context
 from dagster.core.execution.context_creation_pipeline import (
     construct_pipeline_execution_context,
     create_context_creation_data,
@@ -40,6 +40,8 @@ from dagster.core.storage.pipeline_run import PipelineRun
 from dagster.core.types.dagster_type import resolve_dagster_type
 from dagster.core.utility_solids import define_stub_solid
 from dagster.core.utils import make_new_run_id
+from dagster.serdes import ConfigurableClass
+from dagster.utils.backcompat import canonicalize_run_config
 
 # pylint: disable=unused-import
 from ..temp_file import (
@@ -61,15 +63,11 @@ def create_test_pipeline_execution_context(logger_defs=None):
     pipeline_def = PipelineDefinition(
         name='test_legacy_context', solid_defs=[], mode_defs=[mode_def]
     )
-    environment_dict = {'loggers': {key: {} for key in loggers}}
-    pipeline_run = PipelineRun(
-        pipeline_name='test_legacy_context', environment_dict=environment_dict
-    )
+    run_config = {'loggers': {key: {} for key in loggers}}
+    pipeline_run = PipelineRun(pipeline_name='test_legacy_context', run_config=run_config)
     instance = DagsterInstance.ephemeral()
-    execution_plan = create_execution_plan(pipeline=pipeline_def, environment_dict=environment_dict)
-    creation_data = create_context_creation_data(
-        execution_plan, environment_dict, pipeline_run, instance
-    )
+    execution_plan = create_execution_plan(pipeline=pipeline_def, environment_dict=run_config)
+    creation_data = create_context_creation_data(execution_plan, run_config, pipeline_run, instance)
     log_manager = create_log_manager(creation_data)
     scoped_resources_builder = ScopedResourcesBuilder()
     executor_config = create_executor_config(creation_data)
@@ -133,12 +131,12 @@ def execute_solids_within_pipeline(
     pipeline_def,
     solid_names,
     inputs=None,
-    environment_dict=None,
+    run_config=None,
     mode=None,
     preset=None,
     tags=None,
-    run_config=None,
     instance=None,
+    environment_dict=None,
 ):
     '''Execute a set of solids within an existing pipeline.
 
@@ -146,11 +144,11 @@ def execute_solids_within_pipeline(
 
     Args:
         pipeline_def (PipelineDefinition): The pipeline within which to execute the solid.
-        solid_name (str): The name of the solid, or the aliased solid, to execute.
+        solid_names (FrozenSet[str]): A set of the solid names, or the aliased solids, to execute.
         inputs (Optional[Dict[str, Dict[str, Any]]]): A dict keyed on solid names, whose values are
             dicts of input names to input values, used to pass input values to the solids directly.
-            You may also use the ``environment_dict`` to configure any inputs that are configurable.
-        environment_dict (Optional[dict]): The environment configuration that parameterized this
+            You may also use the ``run_config`` to configure any inputs that are configurable.
+        run_config (Optional[dict]): The environment configuration that parameterized this
             execution, as a dict.
         mode (Optional[str]): The name of the pipeline mode to use. You may not set both ``mode``
             and ``preset``.
@@ -158,14 +156,6 @@ def execute_solids_within_pipeline(
             ``mode`` and ``preset``.
         tags (Optional[Dict[str, Any]]): Arbitrary key-value pairs that will be added to pipeline
             logs.
-        run_config (Optional[RunConfig]): Optionally specifies additional config options for
-            pipeline execution.
-
-            Deprecation notice: In 0.8.0, the use of `run_config` to set mode, tags, and step keys
-            will be deprecated. In the interim, if you set a mode using `run_config`, this must
-            match any mode set using `mode` or `preset`. If you set tags using `run_config`, any
-            tags set using `tags` will take precedence. If you set step keys, these must be
-            compatible with any solid subset specified using `preset`.
         instance (Optional[DagsterInstance]): The instance to execute against. If this is ``None``,
             an ephemeral instance will be used, and no artifacts will be persisted from the run.
 
@@ -174,18 +164,20 @@ def execute_solids_within_pipeline(
         executing the solids, keyed by solid name.
     '''
     check.inst_param(pipeline_def, 'pipeline_def', PipelineDefinition)
-    check.list_param(solid_names, 'solid_names', of_type=str)
+    check.set_param(solid_names, 'solid_names', of_type=str)
     inputs = check.opt_dict_param(inputs, 'inputs', key_type=str, value_type=dict)
 
-    sub_pipeline = pipeline_def.build_sub_pipeline(solid_names)
+    # backcompact
+    run_config = canonicalize_run_config(run_config, environment_dict)
+
+    sub_pipeline = pipeline_def.get_pipeline_subset_def(solid_names)
     stubbed_pipeline = build_pipeline_with_input_stubs(sub_pipeline, inputs)
     result = execute_pipeline(
         stubbed_pipeline,
-        environment_dict=environment_dict,
+        run_config=run_config,
         mode=mode,
         preset=preset,
         tags=tags,
-        run_config=run_config,
         instance=instance,
     )
 
@@ -196,12 +188,12 @@ def execute_solid_within_pipeline(
     pipeline_def,
     solid_name,
     inputs=None,
-    environment_dict=None,
+    run_config=None,
     mode=None,
     preset=None,
     tags=None,
-    run_config=None,
     instance=None,
+    environment_dict=None,
 ):
     '''Execute a single solid within an existing pipeline.
 
@@ -211,9 +203,9 @@ def execute_solid_within_pipeline(
         pipeline_def (PipelineDefinition): The pipeline within which to execute the solid.
         solid_name (str): The name of the solid, or the aliased solid, to execute.
         inputs (Optional[Dict[str, Any]]): A dict of input names to input values, used to
-            pass input values to the solid directly. You may also use the ``environment_dict`` to
+            pass input values to the solid directly. You may also use the ``run_config`` to
             configure any inputs that are configurable.
-        environment_dict (Optional[dict]): The environment configuration that parameterized this
+        run_config (Optional[dict]): The environment configuration that parameterized this
             execution, as a dict.
         mode (Optional[str]): The name of the pipeline mode to use. You may not set both ``mode``
             and ``preset``.
@@ -221,14 +213,6 @@ def execute_solid_within_pipeline(
             ``mode`` and ``preset``.
         tags (Optional[Dict[str, Any]]): Arbitrary key-value pairs that will be added to pipeline
             logs.
-        run_config (Optional[RunConfig]): Optionally specifies additional config options for
-            pipeline execution.
-
-            Deprecation notice: In 0.8.0, the use of `run_config` to set mode, tags, and step keys
-            will be deprecated. In the interim, if you set a mode using `run_config`, this must
-            match any mode set using `mode` or `preset`. If you set tags using `run_config`, any
-            tags set using `tags` will take precedence. If you set step keys, these must be
-            compatible with any solid subset specified using `preset`.
         instance (Optional[DagsterInstance]): The instance to execute against. If this is ``None``,
             an ephemeral instance will be used, and no artifacts will be persisted from the run.
 
@@ -236,15 +220,17 @@ def execute_solid_within_pipeline(
         Union[CompositeSolidExecutionResult, SolidExecutionResult]: The result of executing the
         solid.
     '''
+    # backcompact
+    run_config = canonicalize_run_config(run_config, environment_dict)
+
     return execute_solids_within_pipeline(
         pipeline_def,
-        solid_names=[solid_name],
+        solid_names={solid_name},
         inputs={solid_name: inputs} if inputs else None,
-        environment_dict=environment_dict,
+        run_config=run_config,
         mode=mode,
         preset=preset,
         tags=tags,
-        run_config=run_config,
         instance=instance,
     )[solid_name]
 
@@ -256,7 +242,19 @@ def yield_empty_pipeline_context(run_id=None, instance=None):
         instance, 'instance', DagsterInstance, default=DagsterInstance.ephemeral()
     )
     pipeline_run = instance.create_run(
-        run_id=run_id, pipeline_name='<empty>', pipeline_snapshot=None
+        pipeline_name='<empty>',
+        run_id=run_id,
+        environment_dict=None,
+        mode=None,
+        solids_to_execute=None,
+        step_keys_to_execute=None,
+        status=None,
+        tags=None,
+        root_run_id=None,
+        parent_run_id=None,
+        pipeline_snapshot=None,
+        execution_plan_snapshot=None,
+        parent_pipeline_snapshot=None,
     )
     with scoped_pipeline_context(
         create_execution_plan(pipeline), {}, pipeline_run, instance,
@@ -269,8 +267,9 @@ def execute_solid(
     mode_def=None,
     input_values=None,
     tags=None,
-    environment_dict=None,
+    run_config=None,
     raise_on_error=True,
+    environment_dict=None,
 ):
     '''Execute a single solid in an ephemeral pipeline.
 
@@ -282,11 +281,11 @@ def execute_solid(
         mode_def (Optional[ModeDefinition]): The mode within which to execute the solid. Use this
             if, e.g., custom resources, loggers, or executors are desired.
         input_values (Optional[Dict[str, Any]]): A dict of input names to input values, used to
-            pass inputs to the solid directly. You may also use the ``environment_dict`` to
+            pass inputs to the solid directly. You may also use the ``run_config`` to
             configure any inputs that are configurable.
         tags (Optional[Dict[str, Any]]): Arbitrary key-value pairs that will be added to pipeline
             logs.
-        environment_dict (Optional[dict]): The environment configuration that parameterized this
+        run_config (Optional[dict]): The environment configuration that parameterized this
             execution, as a dict.
         raise_on_error (Optional[bool]): Whether or not to raise exceptions when they occur.
             Defaults to ``True``, since this is the most useful behavior in test.
@@ -298,6 +297,8 @@ def execute_solid(
     check.inst_param(solid_def, 'solid_def', ISolidDefinition)
     check.opt_inst_param(mode_def, 'mode_def', ModeDefinition)
     input_values = check.opt_dict_param(input_values, 'input_values', key_type=str)
+    # backcompact
+    run_config = canonicalize_run_config(run_config, environment_dict)
 
     solid_defs = [solid_def]
 
@@ -321,7 +322,7 @@ def execute_solid(
             dependencies=dependencies,
             mode_defs=[mode_def] if mode_def else None,
         ),
-        environment_dict=environment_dict,
+        run_config=run_config,
         mode=mode_def.name if mode_def else None,
         tags=tags,
         raise_on_error=raise_on_error,
@@ -387,42 +388,45 @@ def restore_directory(src):
             shutil.copytree(dst, src)
 
 
-class FilesytemTestScheduler(Scheduler):
+class FilesystemTestScheduler(Scheduler, ConfigurableClass):
     '''This class is used in dagster core and dagster_graphql to test the scheduler's interactions
     with schedule storage, which are implemented in the methods defined on the base Scheduler class.
     Therefore, the following methods used to actually schedule jobs (e.g. create and remove cron jobs
     on a cron tab) are left unimimplemented.
     '''
 
-    def __init__(self, artifacts_dir):
+    def __init__(self, artifacts_dir, inst_data=None):
         check.str_param(artifacts_dir, 'artifacts_dir')
         self._artifacts_dir = artifacts_dir
+        self._inst_data = inst_data
+
+    @property
+    def inst_data(self):
+        return self._inst_data
+
+    @classmethod
+    def config_type(cls):
+        return {'base_dir': str}
+
+    @staticmethod
+    def from_config_value(inst_data, config_value):
+        return FilesystemTestScheduler(artifacts_dir=config_value['base_dir'], inst_data=inst_data)
 
     def debug_info(self):
         return ""
 
-    def start_schedule(self, instance, repository, schedule_name):
+    def start_schedule(self, instance, external_schedule):
         pass
 
-    def stop_schedule(self, instance, repository, schedule_name):
+    def stop_schedule(self, instance, schedule_origin_id):
         pass
 
-    def running_schedule_count(self, repository_name, schedule_name):
+    def running_schedule_count(self, schedule_origin_id):
         return 0
 
-    def get_logs_directory(self, instance, repository, schedule_name):
-        check.inst_param(repository, 'repository', RepositoryDefinition)
-        check.str_param(schedule_name, 'schedule_name')
-        return os.path.join(
-            self._artifacts_dir, repository.name, 'logs', '{}'.format(schedule_name)
-        )
-
-    def get_logs_path(self, instance, repository, schedule_name):
-        check.inst_param(repository, 'repository', RepositoryDefinition)
-        check.str_param(schedule_name, 'schedule_name')
-
-        logs_directory = self.get_logs_directory(instance, repository, schedule_name)
-        return os.path.join(logs_directory, 'scheduler.log')
+    def get_logs_path(self, _instance, schedule_origin_id):
+        check.str_param(schedule_origin_id, 'schedule_origin_id')
+        return os.path.join(self._artifacts_dir, 'logs', schedule_origin_id, 'scheduler.log')
 
     def wipe(self, instance):
         pass
